package com.supwisdom.dlpay.framework.dao;

import com.supwisdom.dlpay.framework.domain.TShopacc;
import org.springframework.data.jpa.repository.*;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.LockModeType;

import javax.persistence.QueryHint;
import java.util.List;

@Repository
public interface ShopaccDao extends JpaRepository<TShopacc, String> {

  @Query(value = "select t.SHOPACCNO from TB_SHOPACC t left join TB_SHOPACCDAY a on t.SHOPACCNO=a.SHOPACCNO and a.accdate=:lastsettday where a.SHOPACCNO is null ", nativeQuery = true)
  List<String> getNewShopacc(@Param("lastsettday") String lastsettday);


  @Query(value = "select shopname from TShopacc where shopaccno=:shopaccno ")
  String getShopname(@Param("shopaccno") String shopaccno);

  @Query(value = "select t.SHOPACCNO as accno from TB_SHOPACC t left join TB_SHOPACCBAL a on t.SHOPACCNO=a.SHOPACCNO where a.SHOPACCNO is null and t.opendate>=:settday ", nativeQuery = true)
  List<String> getNewAddShopacc(@Param("settday") String settday);

  TShopacc findByShopid(Integer shopid);

  @Lock(LockModeType.PESSIMISTIC_WRITE)
  @QueryHints({@QueryHint(name = "javax.persistence.lock.timeout", value = "0")})
  @Query("select a from TShopacc a where a.shopid=?1")
  TShopacc getShopaccWithLockNowait(Integer shopid);

  @Lock(LockModeType.PESSIMISTIC_WRITE)
  @Query("select a from TShopacc a where a.shopid=?1")
  TShopacc getShopaccWithLock(Integer shopid);

  TShopacc getByShopaccno(String shopaccno);

  @Transactional
  @Modifying(clearAutomatically = true)
  @Query("update TShopacc set shopname=?1 where shopaccno=?2")
  void updateShopnameByShopaccno(String shopname, String shopaccno);


  TShopacc findByShopaccno(String acc);
}
